module Hermes
  module Api
    module Dmo
      class EntityReservationsController < Hermes::Api::Dmo::ApplicationController

        class << self
          def mutate_params
            {
              name: { type: 'String'},
              from: { type: 'DateTime'},
              to: { type: 'DateTime'},
              housing_offer_id: { type: 'String'},
              entity_id: { type: 'String'},
            }
          end 
      
          def acceptable_params
            {
              is_active: { type: 'Boolean'},
            }.merge(self.mutate_params).merge(super)
          end 
      
          def valid_operators
            %w[$in $nin $ne $regex]
          end

          def regex_params
            {
              name: { type: 'String'},
            }
          end
        end
        
        def index
          acceptable_params *self.class.acceptable_params.keys
          regex_params *self.class.regex_params.keys
          valid_operators *self.class.valid_operators
          
          paginate(
            index_aggregation,
            each_serializer: @serializer,
            **@options
          )
        end

        def create 
          @instance = @model.new(mutate_valid_params.merge(user_id: @user.id))
          find_and_update_same_instance

          return respond_with_errors(@instance) unless @instance.save
          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end 

        def update
          return render_json_error(:not_found, :instance_not_found) unless @instance

          @instance.assign_attributes(mutate_valid_params.merge(user_id: @user.id))
          find_and_update_same_instance

          super
        end 

        def destroy
          @instance = @model.find(params[:id])
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy
  
          render(body: nil, status: :no_content)
        end

        private 

        def find_and_update_same_instance
          found_instance = Hermes::EntityReservation.order_by(created_at: :desc).find_by(
            entity_id: @instance.entity_id, 
            housing_offer_id: @instance.housing_offer_id,
          )

          if found_instance
            @instance = found_instance
            @instance.assign_attributes(mutate_valid_params.merge(user_id: @user.id))
          end  
        end 

      end
    end
  end
end