require 'rails_helper'
require 'thread'
require 'thwait'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

RSpec.describe UpdateThresholdOffersJob do
  subject { UpdateThresholdOffersJob }
 
  let!(:threshold) do
    create(
      :threshold,
      type: "PLUS",
      codes: ["PLU"],
      thresholds_map: {
        "Paris et communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 46473, 55486, 66017, 74286], "another_person"=>8278},
        "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 42616, 51046, 60429, 68001], "another_person"=>7577},
        "Autres régions"=>{"thresholds_list"=>[20623, 27540, 33119, 39982, 47035, 53008], "another_person"=>5912}
      }
    )
  end
  let!(:offer_type) { create(:housing_offer, threshold_type: 'PLUS', threshold_subtype: 'Z1') }
  let!(:offer_code) { create(:housing_offer, threshold_type: 'PLU', threshold_subtype: 'Z1') }
  let!(:offer_new_type) { create(:housing_offer, threshold_type: 'New Type', threshold_subtype: 'Z1') }
  let!(:offer_new_code) { create(:housing_offer, threshold_type: 'New Code', threshold_subtype: 'Z1') }

  it 'default' do
    expect(offer_type.reload.threshold_id).to eq(threshold.id)
    expect(offer_code.reload.threshold_id).to eq(threshold.id)
    expect(offer_new_type.reload.threshold_id).to eq(nil)
    expect(offer_new_code.reload.threshold_id).to eq(nil)
  end

  it 'should create jobs' do
    expect do
      threshold.update(type: 'New Type')
    end.to change { enqueued_jobs.select{|job| job[:job] == subject }.length }.by(1)
  end

  it 'when changing type' do 
    threshold.update(type: 'New Type')
    execute_last_job(subject)

    expect(offer_type.reload.threshold_id).to eq(nil)
    expect(offer_code.reload.threshold_id).to eq(threshold.id)
    expect(offer_new_type.reload.threshold_id).to eq(threshold.id)
    expect(offer_new_code.reload.threshold_id).to eq(nil)
  end 

  it 'when changing code' do 
    threshold.update(codes: ['New Code'])
    execute_last_job(subject)

    expect(offer_type.reload.threshold_id).to eq(threshold.id)
    expect(offer_code.reload.threshold_id).to eq(nil)
    expect(offer_new_type.reload.threshold_id).to eq(nil)
    expect(offer_new_code.reload.threshold_id).to eq(threshold.id)
  end 

  it 'when changing thresholds_map' do 
    expect(offer_type.reload.ceil_category_1).to eq(23721)
    expect(offer_code.reload.ceil_category_1).to eq(23721)
    expect(offer_new_type.reload.threshold_id).to eq(nil)
    expect(offer_new_code.reload.threshold_id).to eq(nil)

    threshold.update(thresholds_map: {
      "Paris et communes limitrophes"=>{"thresholds_list"=>[10000, 35452, 46473, 55486, 66017, 74286], "another_person"=>8278},
      "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 42616, 51046, 60429, 68001], "another_person"=>7577},
      "Autres régions"=>{"thresholds_list"=>[20623, 27540, 33119, 39982, 47035, 53008], "another_person"=>5912}
    })
    execute_last_job(subject)

    expect(offer_type.reload.ceil_category_1).to eq(10000)
    expect(offer_code.reload.ceil_category_1).to eq(10000)
    expect(offer_new_type.reload.threshold_id).to eq(nil)
    expect(offer_new_code.reload.threshold_id).to eq(nil)
  end 

end
